#*************************************************************
# Progamm:			fluisa_wlgl_stat_2d_quelle
# Sinn des Programms: 		Löser der Wärmeleitungsgleichung
# Sinn der Einheit  :		Hauptprogramm	
# Name dieser Datei:		fluisa_wlgl_stat_2d_quelle_haupt.py
# Versionsnummer:		1
# Wird aufgerufen von:		-
# Ruft auf:			steuer.py, gitter.py, rand_bed.py, koeff_matrix.py, loeser_lgs.py, ausgabe.py
# Nummer im Programmpaket:	1
# von:				7			
# Autor: 			Armin Ulrich
# Erster Lauf: 			22.10.2021
# Letzte Änderung: 		01.01.2023
#*************************************************************

#*************************************************************
# Importieren der Klassen zur Zeitdokumentation
#*************************************************************

import time
import datetime


#*************************************************************
# Importieren der Klassen
#*************************************************************
from steuer import Steuer
from gitter import Gitter
from rand_bed import Rand_bed
from koeff_matrix import Koeff_matrix
from loeser_lgs_5diag import Löser_lgs_5diag
from ausgabe import Ausgabe

#*************************************************************
# Belegung der Klassen
#*************************************************************
steuer = Steuer()
gitter = Gitter()
rand_bed = Rand_bed()
koeff_matrix = Koeff_matrix()
löser_lgs_5diag = Löser_lgs_5diag()
ausgabe = Ausgabe()

#*************************************************************
# Ausgabe an
#*************************************************************
print("Wärmeleitungsgleichung,")
print("2d, stationär")


#*************************************************************
# Berechnung der Startzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Startzeit zu einem String "datum_zeit"
#*************************************************************
dt_start = datetime.datetime.now()
lt_start = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_start[0:6]
datum_zeit = "Datum: " +str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)

#*************************************************************
# Deklarierung Prozeßausgabeliste
#*************************************************************
prozeß_ausgabe_liste = list(range(8))
for i in range(8):
  prozeß_ausgabe_liste[i] = "Nicht durchgeführt!"
  
#*************************************************************
# öffnen der Log-Datei
#*************************************************************
fobj_log = open("wlgl_st_2d.log","w")

#*************************************************************
# Schreiben von Datum und Zeit des Programmstarts 
# in die Logdatei 
#*************************************************************
fobj_log.write("Anfang: "+ "\n" + datum_zeit)
fobj_log.write("\n")
#*************************************************************
# Prüfung, ob Steuerdatei erfolgreich geöffnet wurde
#*************************************************************
if(steuer.steuer_input()):
  fobj_log.write(steuer.rückgabeliste[1] + "\n")
  if(steuer.rückgabeliste[2] == True):
    if(steuer.rückgabeliste[3] == True):
      print(steuer.rückgabeliste[4])
      fobj_log.write(steuer.rückgabeliste[4] + "\n")
    else:  
      print(steuer.rückgabeliste[4])  
      fobj_log.write(steuer.rückgabeliste[4] + "\n")    
#*************************************************************
# Prüfung, ob das Gitter erfolgreich geladen wurde
#*************************************************************
  if gitter.gitter_input(steuer.gitter_datei, steuer.gitter_ascii_bin):
    print("Gitter wurde geladen")
    fobj_log.write("Gitter wurde geladen\n")
#*************************************************************
# Einlesen der Randbedingungen
#*************************************************************
    if rand_bed.rand_bed_input(steuer.rand_datei, gitter.m, gitter.n):
#*************************************************************
# Belegen der Koeffizientenmatrix
#*************************************************************    
      if koeff_matrix.matrix(steuer.lambada,steuer.c,steuer.rho, steuer.s_init, gitter.m, gitter.n, gitter.delta_x, gitter.delta_y, gitter.A_gf_we, gitter.A_gf_sn, gitter.V, \
      rand_bed.rand_art_west, rand_bed.rand_art_ost,rand_bed.rand_art_süd,rand_bed.rand_art_nord,rand_bed.rand_wert_west,rand_bed.rand_wert_ost,rand_bed.rand_wert_süd,rand_bed.rand_wert_nord):
#*************************************************************
# Deklarieren und Initialisieren der Felder im Löser
# eine derartige Trennung verhindert bie instationären Problemen,
# daß in einer Zeitschleife die Felder wiederholt deklariert und initialisiert werden.
#*************************************************************
        if löser_lgs_5diag.feld_init(gitter.m, gitter.n):     

#*************************************************************
# Lösen des GLS der Koeffizientenmatrix
#************************************************************* 
          if löser_lgs_5diag.gs_5diag(gitter.m, gitter.n, steuer.max_iter, steuer.eps, steuer.omega, steuer.resi_max, koeff_matrix.a_s,koeff_matrix.a_w, koeff_matrix.a_p, koeff_matrix.a_e,\
          koeff_matrix.a_n, koeff_matrix.s, steuer.T_init):
            T = löser_lgs_5diag.phi        
#*************************************************************
# Ausgabe der Lösung auf Schirm, in Datei, als Graphik 
#*************************************************************
            if steuer.schirm_ausgabe_wähler : 
              ausgabe.schirm_ausgabe(gitter.m,gitter.n,gitter.x,gitter.y,T) 
            if steuer.datei_ausgabe_wähler: 
              ausgabe.datei_ausgabe(steuer.ausgabe_datei, gitter.m,gitter.n,gitter.x,gitter.y,T )   
            if steuer.graphik_ausgabe_wähler: 
              ausgabe.graphik_ausgabe_3d_T(gitter.m,gitter.n,gitter.x,gitter.y,T,steuer.graphik_datei_ausgabe_wähler,steuer.graphik_3d_datei_name, steuer.graphik_3d_datei_format, \
              steuer.graphik_titel )     
              ausgabe.graphik_ausgabe_kontur_T(gitter.m,gitter.n,gitter.x,gitter.y,T,steuer.graphik_datei_ausgabe_wähler,steuer.graphik_kontur_datei_name, steuer.graphik_kontur_datei_format, \
              steuer.graphik_titel )
          else:
            print("Fehler beim Errechnen der Lösung")    
            fobj_log.write("Fehler beim Errechnen der Lösung\n") 
          prozeß_ausgabe_liste[6] = löser_lgs_5diag.rückgabeliste[1] 
          prozeß_ausgabe_liste[7] = löser_lgs_5diag.rückgabeliste[2]
        else:
          print("Fehler beim Deklarieren des Phi-Feldes im Löser")    
          fobj_log.write("Fehler beim Deklarieren des Phi-Feldes im Löser\n")  
        prozeß_ausgabe_liste[5] = löser_lgs_5diag.rückgabeliste_feld_init[1]                       
      else:
        print("Fehler beim Erstellen der Koeffizientenmatrix")    
        fobj_log.write("Fehler beim Erstellen der Koeffizientenmatrix\n") 
      prozeß_ausgabe_liste[4] = koeff_matrix.rückgabeliste[1]   
    else:
      pass   
    prozeß_ausgabe_liste[3] = rand_bed.rückgabeliste[1]    
  else:
    print("Gitter wurde nicht geladen")
    fobj_log.write( "Gitter wurde nicht geladen\n")
  prozeß_ausgabe_liste[2] = gitter.rückgabeliste[1]
else:
  fobj_log.write( "Datei existiert nicht\n")

prozeß_ausgabe_liste[0] = steuer.rückgabeliste[1]
prozeß_ausgabe_liste[1] = steuer.rückgabeliste[4] 

#*************************************************************
# Berechnung der Endzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Endzeit zu einem String "datum_zeit"
#*************************************************************
lt_end = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_end[0:6]
datum_zeit = "Datum: "+str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)
dt_end = datetime.datetime.now()

#*************************************************************
# Berechnung der Rechendauer mit dem Zeitdifferenzoperator 
# des Moduls datetime
#*************************************************************
dauer = dt_end - dt_start

#*************************************************************
# Schreiben von Datum und Zeit des Programmendes und der 
# Rechendauer in die Logdatei 
#*************************************************************
fobj_log.write("Ende: "+ "\n" +datum_zeit + "\n")
fobj_log.write("Rechendauer: "+ "\n" + str(dauer))

#*************************************************************
# Schließen der Logdatei
#*************************************************************
fobj_log.close()

#*************************************************************
# Importieren der Klasse tkinter für GUI-Elemente
#*************************************************************
import tkinter
main = tkinter.Tk()

#*************************************************************
#Funktion zum Auflösen der Benutzeroberfläche
#*************************************************************
def ende():
  main.destroy()

#*************************************************************
# GUI-Fuktion
#*************************************************************    
             
lb_prozeß_ausgabe_steuer = tkinter.Label(main, text = prozeß_ausgabe_liste[0])
lb_prozeß_ausgabe_steuer.pack()
lb_prozeß_ausgabe_steuer_änderung = tkinter.Label(main, text = prozeß_ausgabe_liste[1])
lb_prozeß_ausgabe_steuer_änderung.pack()
lb_prozeß_ausgabe_gitter = tkinter.Label(main, text = prozeß_ausgabe_liste[2])
lb_prozeß_ausgabe_gitter.pack()
lb_prozeß_ausgabe_rand = tkinter.Label(main, text = prozeß_ausgabe_liste[3])
lb_prozeß_ausgabe_rand.pack()
lb_prozeß_ausgabe_koeff_matrix = tkinter.Label(main, text = prozeß_ausgabe_liste[4])
lb_prozeß_ausgabe_koeff_matrix.pack()

lb_prozeß_ausgabe_löser_phi_init = tkinter.Label(main, text = prozeß_ausgabe_liste[5])
lb_prozeß_ausgabe_löser_phi_init.pack()
lb_prozeß_ausgabe_löser_phi = tkinter.Label(main, text = prozeß_ausgabe_liste[6])
lb_prozeß_ausgabe_löser_phi.pack()
lb_prozeß_ausgabe_ausgabe = tkinter.Label(main, text ="mit " + str(prozeß_ausgabe_liste[7]) + " Iterationen")
lb_prozeß_ausgabe_ausgabe.pack()
#*************************************************************
# Button zur Beendigung des Programms
#*************************************************************   
bende = tkinter.Button(main, text = "Ende",command = ende)
bende.pack()
#*************************************************************
# Endlosschleife
#*************************************************************
main.mainloop()

#*************************************************************
#Programmende
#*************************************************************
